/*
 * Copyright 2013 Andrea "andynaz" Agnesse
 * 
 * This file is part of "iRex DR1000 Merger".
 * 
 * "iRex DR1000 Merger" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * "iRex DR1000 Merger" is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * "iRex DR1000 Merger".  If not, see <http://www.gnu.org/licenses/>.
 * 
 * author e-mail: andynaz@gmail.com
 * author home page: http://andynaz.altervista.org
 * program official page: http://andynaz.altervista.org/Progetti/iRexDR1000Merger.shtm
 */

package it.andynaz.irex.dr1000;

import static java.lang.System.out;
import java.util.*;
import it.andynaz.log.*;
import it.andynaz.utils.Utils;
import it.andynaz.irex.dr1000.document.*;
import java.io.File;

/**
 * Main class.
 * 
 * <p>The scribbles can be taken on a PDF document or with the device built-in
 * notepad. In case of PDF documents, this program creates a new PDF with all
 * its interactive elements, with the scribbles on it. The device must have the
 * version&nbsp;2.0 of the firmware.
 * 
 * <p>To use this program run in a terminal</p>
 * <blockquote><pre>
 * java -jar iRexDR1000Merger.jar -f <i>input_file</i> [-o <i>output_file</i>] [-a <i>action</i>] [-c <i>config_file</i>]
 * </pre></blockquote>
 * <p>Parameter are:</p>
 * <dl>
 *   <dt><b>f</b></dt>
 *   <dd>target PDF or {@code .note} file</dd>
 *   <dt><b>o</b></dt>
 *   <dd>output file (optional; if not present the default one is taken)</dd>
 *   <dt><b>a</b></dt>
 *   <dd>action to be performed (optional; the default one is 'merge')</dd>
 *   <dt><b>c</b></dt>
 *   <dd>path to the a properties file with the configuration of the program
 *   (optional; the default one is a {@code config.properties} file in the
 *   current directory)</dd>
 * </dl>
 * 
 * <p>Possible actions are:</p>
 * <dl>
 *   <dt><b>merge</b></dt>
 *   <dd>default action: merge the scribbles with the PDF in a new PDF file
 *   (default output file is a PDF file in the current folder)</dd>
 *   <dt><b>format</b></dt>
 *   <dd>create a text file with the scribbles data formatted (default output
 *   file is a {@code .dat} file in the current folder)</dd>
 * </dl>
 * 
 * <p>Optionally, you can invoke the program as</p>
 * <blockquote><pre>
 * java -jar iRexDR1000Merger.jar <i>path_to_file</i>
 * </pre></blockquote>
 * <p>in which you specify only the input file (other option use their default
 * value).</p>
 *
 * @author andynaz
 * @version 2013-02-22
 */
public class Merger {

   /**
	 * Takes the arguments from the command line and start the program.
	 * 
	 * @param args the command line arguments
	 * @throws Exception in case of errors
    */
   public static void main(String[] args) throws Exception {
		printHeading();
		
		// --- config.properties
		String configPath = Utils.getInitParam("c", args);
		if( configPath!=null && new File(configPath).exists() )
			Config.init(configPath);
		else
			Config.init();
		
		
		Logger logger = LoggerMgr.getLogger("main");
		logger.log(Arrays.toString(args), Level.FINEST);
		
		// check arguments: at least one
      if( args.length==0 ){
			logger.log("error: no arguments", Level.SEVERE);
			return;
		}
		
		// check if the program has been invoked with the only-one-argument syntax
		boolean oneArgOnly = args.length==1;
		
		// --- input file
		String inputFile = null;
		if( oneArgOnly ){
			inputFile = args[0];
		} else{
			inputFile = Utils.getInitParam("f", args);
			if( inputFile==null ){
				logger.log("error: no input file", Level.SEVERE);
				return;
			}
		}
		logger.log("input file: "+inputFile, Level.DEBUG);
		
		// --- action
		String action = oneArgOnly ? "merge" : Utils.getInitParam("a", args);
		if( action==null ) action="merge";
		logger.log("action: "+action, Level.DEBUG);
		
		// --- output file
		String outputFile = Utils.getInitParam("o", args);
		logger.log("output file: "+outputFile, Level.DEBUG);
		
		Document doc;
		if( inputFile.toLowerCase().endsWith(".pdf") ){
			doc = new PdfDocument(inputFile);
		} else if( inputFile.toLowerCase().endsWith(".note") ){
			doc = new Note(inputFile);
		} else{
			logger.log("error: no PDF file, filePath="+inputFile, Level.SEVERE);
			return;
		}
		
		if( "merge".equals(action)){
			doc.merge(outputFile);
		} else if( "format".equals(action) ){
			doc.format(outputFile);
		}
   }

	/**
	 * Print the program heading.
	 * 
	 * This contains also the licence advice.
	 */
	private static void printHeading() {
		out.println("--------------------------------------------------------------------------------");
		out.println("   iRex DR1000 Merger           Copyright (c) 2013  Andrea \"andynaz\" Agnesse");
		out.println();
		out.println(" This program comes with ABSOLUTELY NO WARRANTY.");
		out.println(" This is free software, and you are welcome to redistribute it under certain");
		out.println(" conditions; read the file 'licence.txt' in the distributed binary for details.");
		out.println("--------------------------------------------------------------------------------");
		out.println();
	}
	
}
