/*
 * Copyright 2013 Andrea "andynaz" Agnesse
 * 
 * This file is part of "iRex DR1000 Merger".
 * 
 * "iRex DR1000 Merger" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * "iRex DR1000 Merger" is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * "iRex DR1000 Merger".  If not, see <http://www.gnu.org/licenses/>.
 * 
 * author e-mail: andynaz@gmail.com
 * author home page: http://andynaz.altervista.org
 * program official page: http://andynaz.altervista.org/Progetti/iRexDR1000Merger.shtm
 */

package it.andynaz.irex.dr1000.document;

import java.io.*;
import java.util.*;
import com.lowagie.text.Rectangle;
import com.lowagie.text.pdf.*;
import it.andynaz.irex.dr1000.scribble.*;
import it.andynaz.log.*;
import it.andynaz.utils.Utils;


/**
 * Represent a DR1000 PDF file.
 * 
 * An object of that kind is associated with a path to a PDF file.
 *
 * @author andynaz
 * @version 2013-02-22
 */
public class PdfDocument extends Document{
	/**
	 * Name of the default SQLite database file for PDF documents.
	 */
	public static final String DB_DEFAULT_NAME = "metadata.db";
	
	/**
	 * Source PDF file.
	 */
	private File file;
	
	
	/**
	 * Creates a PDF document.
	 * 
	 * @param filePath path to the PDF file
	 * 
	 * @throws IllegalArgumentException if {@code filePath} does not represent a
	 * file
	 */
	public PdfDocument(String filePath){
		this(filePath, new File(new File(filePath).getParentFile(), DB_DEFAULT_NAME));
	}
	
	/**
	 * Creates a PDF document.
	 * 
	 * @param filePath path to the PDF file
	 * @param db path to the database file
	 * 
	 * @throws IllegalArgumentException if {@code filePath} does not represent a
	 * file
	 */
	public PdfDocument(String filePath, File db){
		super(db);
		File file = new File(filePath);
		if( !file.isFile() )
			throw new IllegalArgumentException("error: input file is not a file, filePath="+filePath);
		this.file = file;
	}
	

	/**
	 * Return the name of the PDF file.
	 *
	 * @return the name of the PDF file
	 */
	@Override
	protected String getFileName(){
		return this.file.getName();
	}
	
	/**
	 * Return the name of the PDF source file, without the extension.
	 * 
	 * @return the name of the PDF source file, without the extension
	 */
	@Override
	protected String getDefaultFileName(){
		String fileName = this.file.getName();
		return (fileName.indexOf('.')==-1 ? fileName : fileName.substring(0, fileName.lastIndexOf('.')));
	}

	/**
	 * Merge the scribbles with the PDF.
	 * 
	 * The output is written in the file specified by the 'output' argument.
	 * 
	 * <p>If the file is not found in the database, the program list all the
	 * files in the database having at least a scribble, so that user can choose
	 * which is the corresponding one. This can happen with a file having a name
	 * with all uppercases character, as the Linux operating system of the DR1000
	 * may not recognize them properly.</p>
	 * 
	 * @param outputFile the output file path; if it is {@code null} or empty,
	 * the default one returned by {@link #getDefaultFileName()} is used
	 */
	@Override
	public void merge(String outputFile){
		if( Utils.isEmpty(outputFile) )
			outputFile = getDefaultFileName()+".pdf";
		
		try{
			ScribbleMgr sm = getScribbleMgr();

			int idFile = sm.getFileId(file.getName());
			if( idFile==0 ) return;

			List<Integer> scribbledPages = sm.getScribbledPages(idFile);
			PdfReader reader = new PdfReader(file.getPath());
			PdfStamper stamp = new PdfStamper(reader, new FileOutputStream(outputFile));
			for( int pageNum=1; pageNum<=reader.getNumberOfPages(); pageNum++ ){
				if( scribbledPages.contains(pageNum) ){
					List<Integer> blob = sm.getScribbles(pageNum, idFile);
					Page p = Page.createSCribblePage(blob);
					PdfContentByte cbOver = stamp.getOverContent(pageNum);
					Rectangle rectCropBox = reader.getCropBox(pageNum);
					int nRotation = reader.getPageRotation(pageNum);
					while (nRotation > 0) {
						rectCropBox = rectCropBox.rotate();
						nRotation -= 90;
					}
					if( p!=null )
						p.drawStrokes(cbOver, rectCropBox.left(), rectCropBox.top());
				}
			}
			stamp.close();
			sm.closeConnection();
		} catch(Exception e){
			LoggerMgr.getLogger("merge").log(e, Level.WARNING);
		}
	}
	
}
