/*
 * Copyright 2013 Andrea "andynaz" Agnesse
 * 
 * This file is part of "iRex DR1000 Merger".
 * 
 * "iRex DR1000 Merger" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * "iRex DR1000 Merger" is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * "iRex DR1000 Merger".  If not, see <http://www.gnu.org/licenses/>.
 * 
 * author e-mail: andynaz@gmail.com
 * author home page: http://andynaz.altervista.org
 * program official page: http://andynaz.altervista.org/Progetti/iRexDR1000Merger.shtm
 */

package it.andynaz.irex.dr1000.scribble;

import java.io.*;
import java.util.*;
import com.lowagie.text.pdf.PdfContentByte;

/**
 * Represent a page of scribbles.
 * 
 * The scribbles are organized as a list of {@link Stroke} objects, which have a
 * list of {@link Stroke.Point} objects.
 * 
 * <p>Scribbles are stored in the {@code annotations.data} column (see
 * documentation of class {@link it.andynaz.irex.dr1000.document.ScribbleMgr}),
 * as a binary blob with the following structure. The not-used parts are
 * represented in gray (in this case the meaning is not sure) and the bytes in
 * the examples  are represented as a couple of hexadecimal digits. Note that
 * the bytes are stored in a reversed order, so that the pair of bytes "01 00"
 * represent the integer 1 (and not 256)</p>
 * 
 * <p>The first 40 bytes are the ones related to the page data:</p>
 * <table border="1">
 * <tr>                        <th>length</th> <th>type</th> <th>meaning</th>           <th>example</th></tr>
 * <tr style="color:#aaaaaa;"> <td>2</td>      <td></td>     <td>version</td>           <td>01 00</td></tr>
 * <tr style="color:#aaaaaa;"> <td>4</td>      <td></td>     <td>orientation</td>       <td>00 00 00 00</td></tr>
 * <tr style="color:#aaaaaa;"> <td>8</td>      <td></td>     <td>page_pixel</td>        <td>00 00 00 00 00 00 00 00</td> </tr>
 * <tr style="color:#aaaaaa;"> <td>2</td>      <td></td>     <td>page_r</td>            <td>00 00</td></tr>
 * <tr style="color:#aaaaaa;"> <td>2</td>      <td></td>     <td>page_g</td>            <td>00 00</td></tr>
 * <tr style="color:#aaaaaa;"> <td>2</td>      <td></td>     <td>page_b</td>            <td>00 00</td></tr>
 * <tr style="color:#aaaaaa;"> <td>4</td>      <td></td>     <td>ul_x</td>              <td>00 00 00 00</td></tr>
 * <tr style="color:#aaaaaa;"> <td>4</td>      <td></td>     <td>ul_y</td>              <td>00 00 00 00</td></tr>
 * <tr style="color:#aaaaaa;"> <td>4</td>      <td></td>     <td>lr_x</td>              <td>D7 04 00 00</td></tr>
 * <tr style="color:#aaaaaa;"> <td>4</td>      <td></td>     <td>lr_y</td>              <td>DA 06 00 00</td></tr>
 * <tr>                        <td>4</td>      <td>int</td>  <td>number of strokes</td> <td>A3 00 00 00</td></tr>
 * </table>
 * <p>Only the last one is actually used, as it indicates the number of strokes
 * in the page.</p>
 * 
 * <p>After that 40 bytes, there is a sequence of 24 bytes representing the
 * properties of a stroke, according to the following schema.</p>
 * <table border="1">
 * <tr>                        <th>length</th> <th>type</th>  <th>meaning</th>          <th>example</th>     </tr>
 * <tr style="color:#aaaaaa;"> <td>4</td>      <td></td>      <td>color</td>            <td>03 00 00 00</td> </tr>
 * <tr>                        <td>4</td>      <td>float</td> <td>zoom</td>             <td>D7 A0 89 42</td> </tr>
 * <tr style="color:#aaaaaa;"> <td>4</td>      <td></td>      <td>layer</td>            <td>00 00 00 00</td> </tr>
 * <tr style="color:#aaaaaa;"> <td>4</td>      <td></td>      <td>shape</td>            <td>00 00 00 00</td> </tr>
 * <tr>                        <td>4</td>      <td>int</td>   <td>width</td>            <td>01 00 00 00</td> </tr>
 * <tr>                        <td>4</td>      <td>int</td>   <td>number of points</td> <td>35 00 00 00</td> </tr>
 * </table>
 * 
 * <p>After that part, there is a sequence of 12-bytes blocks, each representing
 * a point, according to the following schema.</p>
 * <table border="1">
 * <tr>                        <th>length</th> <th>type</th>  <th>meaning</th>      <th>example</th>     </tr>
 * <tr>                        <td>4</td>      <td>int</td>   <td>x coordinate</td> <td>48 00 00 00</td> </tr>
 * <tr>                        <td>4</td>      <td>int</td>   <td>y coordinate</td> <td>6C 00 00 00</td> </tr>
 * <tr style="color:#aaaaaa;"> <td>4</td>      <td></td>      <td>pressure</td>     <td>00 00 00 00</td> </tr>
 * </table>
 * 
 * <p>After the sequence of points, another stroke  begins (or the end of the
 * blob is reached).</p>
 * 
 * <p>To create a {@code Page} object from a blob, use the {@link
 * Page#createSCribblePage(java.util.List) createSCribblePage} method.</p>
 *
 * @author andynaz
 * @version 2013-02-22
 */
public class Page {
	/**
	 * List of strokes.
	 * 
	 * Contains all the strokes in the page.
	 */
	private List<Stroke> lStroke = new ArrayList<Stroke>();

	/**
	 * Creates a new Page object.
	 */
	public Page(){ }

	/**
	 * Add a stroke.
	 * 
	 * The stroke is added only if it is not null.
	 * 
	 * @param s the stroke to add
	 */
	public void addStroke(Stroke s){
		if( s!=null)
			lStroke.add(s);
	}

	/**
	 * Draw all the strokes of this page.
	 * 
	 * @param cb the content of the PDF
	 * @param xAdjust the parameter to adjust the 'x' coordinates of the strokes
	 * @param yAdjust the parameter to adjust the 'y' coordinates of the strokes
	 */
	public void drawStrokes(PdfContentByte cb, float xAdjust, float yAdjust) {
		for( Stroke stroke : lStroke ){
			stroke.draw(cb, xAdjust, yAdjust);
		}
	}

	/**
	 * Create a Page from the blob.
	 *
	 * This method also creates all the {@link Stroke} objects inside the Page.
	 *
	 * @param l the blob
	 * @return a Page with all the scribbles in the blob, or {@code null} if the
	 * list {@code l} is empty
	 */
	public static Page createSCribblePage(List<Integer> l) {
		Page p = null;
		if (l == null || l.isEmpty()) {
			return p;
		}
		p = new Page();
		// ----- page -----                                                                  // ----- page -----
		Util.skip(l, 36);                                                                         // 36: useless things
		int nStroke = Util.getIntFromBytes(l.remove(0), l.remove(0), l.remove(0), l.remove(0));   //4: number of strokes
		for (int c = 0; c < nStroke; c++) {
			Stroke s = new Stroke();
		// ----- stroke -----                                                                // ----- stroke -----
			Util.skip(l, 4);                                                                       //4: color
			float zoom = Util.get4ByteFloat(l.remove(0), l.remove(0), l.remove(0), l.remove(0));   //4: zoom
			s.setZoom(zoom);
			Util.skip(l, 8);                                                                       //8: layer+shape
			int width = Util.getIntFromBytes(l.remove(0), l.remove(0), l.remove(0), l.remove(0));  //4: width
			s.setWidth(width);
			int nPoint = Util.getIntFromBytes(l.remove(0), l.remove(0), l.remove(0), l.remove(0)); //4: number of points
			for (int j = 0; j < nPoint; j++) {
				assert l.size() >= 12 : "size=" + l.size();
		// ----- point -----                                                                 // ----- point -----
				int x = Util.getIntFromBytes(l.remove(0), l.remove(0), l.remove(0), l.remove(0));   //4: x
				int y = Util.getIntFromBytes(l.remove(0), l.remove(0), l.remove(0), l.remove(0));   //4: y
				s.addPoint(x, y);
				Util.skip(l, 4);                                                                    //4: pressure
			}
			p.addStroke(s);
		}
		assert l.isEmpty() : "lista non finita";
		return p;
	}

}
