/*
 * Copyright 2013 Andrea "andynaz" Agnesse
 * 
 * This file is part of "iRex DR1000 Merger".
 * 
 * "iRex DR1000 Merger" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * "iRex DR1000 Merger" is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * "iRex DR1000 Merger".  If not, see <http://www.gnu.org/licenses/>.
 * 
 * author e-mail: andynaz@gmail.com
 * author home page: http://andynaz.altervista.org
 * program official page: http://andynaz.altervista.org/Progetti/iRexDR1000Merger.shtm
 */

package it.andynaz.irex.dr1000.scribble;

import com.lowagie.text.pdf.PdfContentByte;
import java.io.*;
import java.util.*;

/**
 * A stroke.
 *
 * A stroke is made up of various points. Points can be added via the {@link
 * #addPoint(int, int)} method and are memorized in a ordered list.<br/>
 * A stroke has also a width and a zoom level.
 *
 * @author andynaz
 * @version 2013-02-22
 */
public class Stroke {
	
	/**
	 * List of points.
	 */
	private List<Point> lPoint = new ArrayList<Point>();
	
	/**
	 * Zoom level.
	 */
	private float zoom;
	
	/**
	 * Width.
	 */
	private int width;
	

	/**
	 * Create a stroke.
	 */
	public Stroke(){ }
	

	/**
	 * Set the width of this stroke.
	 * 
	 * @param width the width of the stroke (must be positive)
	 * 
	 * @throws IllegalArgumentException if {@code width} is not positive
	 */
	public void setWidth(int width) {
		if( width<=0 )
			throw new IllegalArgumentException("non-positive width");
		this.width = width;
	}

	/**
	 * Set the zoom level of this stroke.
	 * 
	 * @param zoom the zoom level of the stroke (must be positive)
	 * 
	 * @throws IllegalArgumentException if {@code zoom} is not positive
	 */
	public void setZoom(float zoom) {
		if( zoom<=0 )
			throw new IllegalArgumentException("non-positive zoom level");
		this.zoom = zoom;
	}

	/**
	 * Add a point to this stroke.
	 * 
	 * the point is specified with its coordinates.
	 *
	 * @param x 'x' coordinate of the point to add
	 * @param y 'y' coordinate of the point to add
	 */
	public void addPoint(int x, int y){
		Point p = new Point(x, y);
		lPoint.add(p);
	}

	/**
	 * Draw the stroke.
	 * 
	 * <p>When being drawn on the PDF, all the points of the stroke are scaled
	 * and adapted for the iText library, which uses a different coordinate
	 * system as the one used by the iPDF library, as shown in the following
	 * picture.<p>
	 * 
	 * <p><img src="doc-files/coordinate.png" /></p>
	 * 
	 * <p>At first are scaled with the zoom factor using (as explained <a
	 * target="_balnk" href="http://www.mobileread.com/forums/showpost.php?p=868622&postcount=13">here</a>)
	 * the formulas
	 * <pre>
	 *          x                1            48 x
	 * x' = ---------- * ----------------- = ------
	 *       zoom/100     160/72 (1-1/16)     zoom
	 * 
	 *          y                1            48 y
	 * y' = ---------- * ----------------- = ------
	 *       zoom/100     160/72 (1-1/16)     zoom
	 * </pre>
	 * so the zoom factor has to be set with the {@link #setZoom(float)} method
	 * before calling the {@link #draw(com.lowagie.text.pdf.PdfContentByte,
	 * float, float)} method).<br/>
	 * After scaling, the coordinates are converted to the itext coordinates
	 * system using the formulas
	 * <pre>
	 * x' = x<sub>a</sub> + x
	 * 
	 * y' = y<sub>a</sub> - y
	 * </pre>
	 * where <code>x<sub>a</sub></code> and <code>y<sub>a</sub></code> are the
	 * arguments of the method.</p>
	 * 
	 * @param cb content of the PDF
	 * @param xAdjust value to correct the 'x' coordinate of all the points in
	 * this stroke
	 * @param yAdjust value to correct the 'y' coordinate of all the points in
	 * this stroke
	 * 
	 * @throws ArithmeticException if the {@link #setZoom(float)}} method has not
	 * been called
	 */
	void draw(PdfContentByte cb, float xAdjust, float yAdjust) {
		if (lPoint.size()<2)
			return;
      
		// also the line width is scaled
		cb.setLineWidth( (720*width)/(zoom*15) );

		float x,y;
		x = xAdjust + lPoint.get(0).getXScaledValue();
		y = yAdjust - lPoint.get(0).getYScaledValue();
		cb.moveTo(x, y);
		for (int pointNumber = 1; pointNumber<lPoint.size(); pointNumber++) {
			x = xAdjust + lPoint.get(pointNumber).getXScaledValue();
			y = yAdjust - lPoint.get(pointNumber).getYScaledValue();
			cb.lineTo(x, y);
		}

		cb.stroke();
	}

	/**
	 * A point.
	 *
	 * A point is made of two int coordinates, 'x' and 'y'.
	 */
	private class Point {
		int x;
		int y;
		
		/**
		 * Creates a new point.
		 * 
		 * @param x 'x' coordinate of the point
		 * @param y 'y' coordinate of the point
		 */
		Point(int x, int y) {
			this.x = x;
			this.y = y;
		}
		
		/**
		 * Return the scaled value of the 'y' coordinate of this point.
		 * 
		 * The value is calculated considering the zoom level of the stroke.
		 * 
		 * @return the scaled 'y' value of this point
		 */
		float getXScaledValue(){
			return 48*this.x/zoom;
		}
		
		/**
		 * Return the scaled value of the 'y' coordinate of this point.
		 * 
		 * The value is calculated considering the zoom level of the stroke.
		 * 
		 * @return the scaled 'y' value of this point
		 */
		float getYScaledValue(){
			return 48*this.y/zoom;
		}
	}
	
}
